@@ -25,7 +25,7 @@ struct got_pack {
 	char *path_packfile;
 	int fd;
 	uint8_t *map;
-	size_t filesize;
+	off_t filesize;
 	struct got_pack_privsep_child *privsep_child;
 	int basefd;
 	int accumfd;
@@ -41,7 +41,7 @@ const struct got_error *got_pack_start_privsep_child(struct got_pack *,
 const struct got_error *got_pack_close(struct got_pack *);
 
 const struct got_error *got_pack_parse_offset_delta(off_t *, size_t *,
-    struct got_pack *, off_t, int);
+    struct got_pack *, off_t, size_t);
 const struct got_error *got_pack_parse_ref_delta(struct got_object_id *,
     struct got_pack *, off_t, int);
 const struct got_error *got_pack_resolve_delta_chain(struct got_delta_chain *,
