@@ -777,20 +777,29 @@ recv_done(int *packfd, int outfd, struct imsgbuf *ibuf, int chattygot)
 	if (err)
 		return err;
 
-	err = gotd_imsg_poll_recv(&imsg, ibuf, 0);
-	if (err)
-		return err;
+	while (*packfd == -1 && err == NULL) {
+		err = gotd_imsg_poll_recv(&imsg, ibuf, 0);
+		if (err)
+			break;
 
-	if (imsg.hdr.type == GOTD_IMSG_ERROR)
-		err = gotd_imsg_recv_error(NULL, &imsg);
-	else if (imsg.hdr.type != GOTD_IMSG_PACKFILE_PIPE)
-		err = got_error(GOT_ERR_PRIVSEP_MSG);
-	else if (imsg.fd == -1)
-		err = got_error(GOT_ERR_PRIVSEP_NO_FD);
-	if (err == NULL)
-		*packfd = imsg.fd;
+		switch (imsg.hdr.type) {
+		case GOTD_IMSG_ERROR:
+			err = gotd_imsg_recv_error(NULL, &imsg);
+			break;
+		case GOTD_IMSG_PACKFILE_PIPE:
+			if (imsg.fd != -1)
+				*packfd = imsg.fd;
+			else
+				err = got_error(GOT_ERR_PRIVSEP_NO_FD);
+			break;
+		default:
+			err = got_error(GOT_ERR_PRIVSEP_MSG);
+			break;
+		}
+
+		imsg_free(&imsg);
+	}
 
-	imsg_free(&imsg);
 	return err;
 }
 
@@ -962,7 +971,7 @@ serve_read(int infd, int outfd, int gotd_sock, const char *repo_path,
 		pack_chunksize = sizeof(buf);
 
 	for (;;) {
-		ssize_t r, w;
+		ssize_t r;
 
 		r = read(packfd, use_sidebands ? &buf[1] : buf,
 		    pack_chunksize);
@@ -980,12 +989,10 @@ serve_read(int infd, int outfd, int gotd_sock, const char *repo_path,
 			if (err)
 				break;
 		} else {
-			w = write(outfd, buf, r);
-			if (w == -1) {
-				err = got_error_from_errno("write");
-				break;
-			} else if (w != r) {
-				err = got_error(GOT_ERR_IO);
+			err = got_poll_write_full(outfd, buf, r);
+			if (err) {
+				if (err->code == GOT_ERR_EOF)
+					err = NULL;
 				break;
 			}
 		}
